#include "hankel_transf.h"



void DFTYieldExpon(double *ht_vec, double *td_vec, int *length_td){

  int i, j, k, n;
  double incr_suggest, incr_desired;
  int num_samp_suggest_no_endpt, num_samp_desired_no_endpt, num_samp_desired;
  double tmp, min_s, max_s, min_x, max_x, cos_part, sum;
  double *s, *x;
  gsl_matrix *f_val_mat;


  /* Let n=*length_td */
  n = *length_td;


  /* ********************************** */
  /* ENUMERATE AND EXAMINE ALL VALUES OF S */

  /* Make list of all s's */
  s = calloc(n, sizeof(double));
  for (i=0; i<n; i++)
    s[i] = td_vec[i]/sqrt(2.0);


  /* Find max and min of s. */
  min_s = s[0];
  max_s = s[0];
  for (i=1; i<n; i++){
    if (s[i] > max_s)
      max_s = s[i];
    else if (s[i] < min_s)
      min_s = s[i];
  }
  /* ********************************** */



  /* ********************************** */
  /* ENUMERATE VALUES OF X */

  /* Determine the range of the x's */
  max_x = 1.5 * (1.0/(2.0*min_s));
  min_x = -max_x;

  /* Determine the suggested increment for the x's */
  incr_suggest = (2.0/3.0) * (1.0/(2.0*max_s));

  /* Determine the desired increment of x */
  num_samp_suggest_no_endpt = ceil((max_x - min_x) / incr_suggest);
  num_samp_desired_no_endpt = num_samp_suggest_no_endpt + 1;
  incr_desired = (max_x - min_x) / num_samp_desired_no_endpt;
  num_samp_desired = num_samp_desired_no_endpt + 1;

  /* Find all combos of x1 and x2 and evaluate function. */
  x = calloc(num_samp_desired, sizeof(double));
  for (i=0; i<num_samp_desired; i++){
    if (i==0)
      x[i] = min_x;
    else
      x[i] = incr_desired + x[i-1];
  }
  /* ********************************** */



  /* ********************************** */
  /* FOR EACH VALUE OF X, EVALUATE FUNCTION TO BE TRANSFORMED */

  f_val_mat = gsl_matrix_alloc(num_samp_desired, num_samp_desired);
  for (i=0; i<num_samp_desired; i++)
    for (j=0; j<num_samp_desired; j++)
      gsl_matrix_set(f_val_mat, i, j, eval_f(x[i], x[j]));
  /* ********************************** */



  /* ********************************** */
  /* FOR EACH S VALUE, APPROXIMATE INTEGRAL BY SUMMING OVER X. */

  for (i=0; i<n; i++){    /* Loop through s */

    sum = 0.0;

    for (j=0; j<num_samp_desired; j++){
      for (k=0; k<num_samp_desired; k++){
	cos_part = cos( 2.0 * M_PI * ((x[j]*s[i]) + (x[k]*s[i])) );
	sum = sum + (gsl_matrix_get(f_val_mat, j, k) * cos_part);
      }
    }

    ht_vec[i] = sum;
  }
  /* ********************************** */


  /* Free memory */
  gsl_matrix_free(f_val_mat);
  free(s);
  free(x);

  return;
}



double eval_f(double x1, double x2){

  double r_part, denom_bef_power;

  r_part = (x1*x1) + (x2*x2);
  denom_bef_power = (4.0 * M_PI * M_PI * r_part) + 1.0;

  return( pow(denom_bef_power, -0.75) );
}
