#include "hankel_transf.h"



void DFTYieldExpon(double *ht_vec, double *td_vec, int *length_td){

  int i, j, k, n;
  double incr_suggest, incr_desired;
  int num_samp_suggest_no_endpt, num_samp_desired_no_endpt, num_samp_desired;
  double tmp, min_s, max_s, min_x, max_x, cos_part, sum;
  double *s, *x;
  gsl_matrix *f_val_mat;


  /* Let n=*length_td */
  n = *length_td;


  /* ********************************** */
  /* ENUMERATE AND EXAMINE ALL VALUES OF S */

  /* Make list of all s's */
  s = calloc(n, sizeof(double));
  for (i=0; i<n; i++)
    s[i] = td_vec[i]/sqrt(2.0);


  /* Find max and min of s. */
  min_s = s[0];
  max_s = s[0];
  for (i=1; i<n; i++){
    if (s[i] > max_s)
      max_s = s[i];
    else if (s[i] < min_s)
      min_s = s[i];
  }
  printf("\nmin_s=%lf, max_s=%lf\n", min_s, max_s);
  /* ********************************** */



  /* ********************************** */
  /* ENUMERATE VALUES OF X */

  /* Determine the range of the x's */
  max_x = 2.0 * (1.0/(2.0*min_s));
  min_x = -max_x;

  /* Determine the suggested increment for the x's */
  incr_suggest = 0.5 * (1.0/(2.0*max_s));

  /* Determine the desired increment of x */
  num_samp_suggest_no_endpt = ceil((max_x - min_x) / incr_suggest);
  num_samp_desired_no_endpt = num_samp_suggest_no_endpt + 1;
  incr_desired = (max_x - min_x) / num_samp_desired_no_endpt;
  num_samp_desired = num_samp_desired_no_endpt + 1;

  /* Find all combos of x1 and x2 and evaluate function. */
  x = calloc(num_samp_desired, sizeof(double));
  for (i=0; i<num_samp_desired; i++){
    if (i==0)
      x[i] = min_x;
    else
      x[i] = incr_desired + x[i-1];
  }
  printf("min_x=%lf, max_x=%lf\n", min_x, max_x);
  printf("incr_desired=%lf, num_samp_desired=%d\n", incr_desired, num_samp_desired);
  /* ********************************** */



  /* ********************************** */
  /* FOR EACH VALUE OF X, EVALUATE FUNCTION TO BE TRANSFORMED */

  f_val_mat = gsl_matrix_alloc(num_samp_desired, num_samp_desired);
  for (i=0; i<num_samp_desired; i++)
    for (j=0; j<num_samp_desired; j++)
      gsl_matrix_set(f_val_mat, i, j, eval_f(x[i], x[j]));
  /* ********************************** */



  /* ********************************** */
  /* FOR EACH S VALUE, APPROXIMATE INTEGRAL BY SUMMING OVER X. */

  for (i=0; i<n; i++){    /* Loop through s */

    sum = 0.0;

    for (j=0; j<num_samp_desired; j++){
      for (k=0; k<num_samp_desired; k++){
	cos_part = cos( 2.0 * M_PI * ((x[j]*s[i]) + (x[k]*s[i])) );
	sum = sum + (gsl_matrix_get(f_val_mat, j, k) * cos_part);
      }
    }

    ht_vec[i] = sum;
  }
  /* ********************************** */


  /* Free memory */
  gsl_matrix_free(f_val_mat);
  free(s);
  free(x);

  return;
}



double approx_with_DFT(double s1, double s2, double min_val, double max_val,
			  double incr){

  double x1, x2;
  double value_of_f, cos_part, const_part;
  double sum=0.0, sum_F0=0.0;
  double adj_sum, adj_sum_F0;


  x1 = min_val;
  x2 = min_val;

  while (x1 <= max_val){

    while (x2 <= max_val){
      value_of_f = eval_f(x1, x2);

      /* Since for s=0, we have cos(0)=1 */
      sum_F0 = sum_F0 + value_of_f;

      /* When s!=0, must include cos part */
      cos_part = cos( 2.0 * M_PI * ((x1*s1) + (x2*s2)) );
      sum = sum + (value_of_f * cos_part);

      x2 = x2 + incr;
    }

    x1 = x1 + incr;
  }


  /* Constant part (part in front of the integral) is 2*pi*sqrt(2*pi). */
  const_part = sqrt(2 * M_PI);

  /* The volume of each approx. box is the approximate height of
     the curve at (x1,x2) times the area of the box's base. */
  adj_sum = const_part * sum * (incr*incr);
  adj_sum_F0 = const_part * sum_F0 * (incr*incr);

  return(adj_sum/sqrt(adj_sum_F0));
}



void oldDFTYieldExpon(double *ht_vec, double *td_vec, int *length_td){

  int i, n;
  double incr_suggest, incr_desired;
  double num_samp_suggest_no_endpt, num_samp_desired_no_endpt;
  double min_val, max_val, incr;
  double s;

  /* Let n=*length_td */
  n = *length_td;

  for (i=0; i<n; i++){
    /* s = td_vec[i]/sqrt(2.0); */
    s = td_vec[i];

    min_val = -8.0 * (1.0/(2.0*s));
    max_val = -min_val;
    incr_suggest = 0.125 * (1.0/(2.0*s));

    num_samp_suggest_no_endpt = (max_val-min_val)/incr_suggest;
    num_samp_desired_no_endpt = num_samp_suggest_no_endpt + 1;
    incr_desired = (max_val - min_val) / num_samp_desired_no_endpt;

    /* printf("\nmin_val=%lf, max_val=%lf, incr_desired=%lf\n", min_val, max_val, incr_desired); */

    ht_vec[i] = approx_with_DFT(s, 0, min_val, max_val, incr_desired);
  }

  return;
}



double old_approx_with_DFT(double s1, double s2, double min_val,
			   double max_val, double incr){

  double x1, x2;
  double value_of_f, cos_part, const_part;
  double sum=0.0, sum_F0=0.0;
  double adj_sum, adj_sum_F0;


  x1 = min_val;
  x2 = min_val;

  while (x1 <= max_val){

    while (x2 <= max_val){
      value_of_f = eval_f(x1, x2);

      /* Since for s=0, we have cos(0)=1 */
      sum_F0 = sum_F0 + value_of_f;

      /* When s!=0, must include cos part */
      cos_part = cos( 2.0 * M_PI * ((x1*s1) + (x2*s2)) );
      sum = sum + (value_of_f * cos_part);

      x2 = x2 + incr;
    }

    x1 = x1 + incr;
  }


  /* Constant part (part in front of the integral) is 2*pi*sqrt(2*pi). */
  const_part = sqrt(2 * M_PI);

  /* The volume of each approx. box is the approximate height of
     the curve at (x1,x2) times the area of the box's base. */
  adj_sum = const_part * sum * (incr*incr);
  adj_sum_F0 = const_part * sum_F0 * (incr*incr);

  return(adj_sum/sqrt(adj_sum_F0));
}



double eval_integral(double s1, double s2, double x1, double x2){

  double cos_part;

  cos_part = cos( 2.0 * M_PI * ((x1*s1) + (x2*s2)) );
  return(eval_f(x1, x2) * cos_part);
}



double eval_f(double x1, double x2){

  double r_part, denom_bef_power;

  r_part = (x1*x1) + (x2*x2);
  denom_bef_power = (4.0 * M_PI * M_PI * r_part) + 1.0;

  return( pow(denom_bef_power, -0.75) );
}







void HankelYieldExpon(double *ht_vec, double *td_vec, int *length_td){

  int i, n;
  double min_val, max_val, incr;
  double num_cycles=50.0;
  double num_samps_per_cycle=8.0;


  /* Let n=*length_td */
  n = *length_td;


  for (i=0; i<n; i++){
    min_val = incr;
    max_val = 8.0/td_vec[i];
    incr = 1.0/(32.0*td_vec[i]);
    ht_vec[i] = approx_with_hankel(td_vec[i], min_val, max_val, incr);
  }


  /* Make call to the function that does the work. */
/*   for (i=0; i<n; i++){ */
/*     min_val = 0.01; */
/*     max_val = num_cycles/td_vec[i]; */
/*     incr = 1.0/(num_samps_per_cycle*td_vec[i]); */
/*     ht_vec[i] = old_approx_with_hankel(td_vec[i], min_val, max_val, incr); */
/*   } */

  return;
}



double approx_with_hankel(double td, double min_val, double max_val,
			  double incr){

  double x;
  double func_to_transf, eval_bessel, curve_ht, const_part;
  double sum=0.0;


  x = min_val;

  while (x <= max_val){
    
    /* The area of our approx. rectangle is the approximate height of
       the curve at x times the incr, which is the with of our
       approx. rectangle. */
    sum = sum + (eval_product(td, x) * incr);
    
    /* Increment the value of x for the next time through the loop. */
    x = x + incr;
  }


  /* Constant part (part in front of the integral) is 2*pi*sqrt(2*pi). */
  const_part = 2.0 * M_PI * sqrt(2 * M_PI);

  return(const_part * sum);
}



double old_approx_with_hankel(double td, double min_val, double max_val,
			      double incr){

  double x;
  double func_to_transf, eval_bessel, curve_ht, const_part;
  double bridge_val=10.0, bridge_incr=0.02, sum=0.0;


  /* No matter the value of td, we're still going to take a lot of
     sameples from the area around 1/(sqrt(2)*pi), where the value of
     the function to transform reaches its maximum. */
  x = bridge_incr;

  while (x <= bridge_val){
    
    /* The area of our approx. rectangle is the approximate height of
       the curve at x times the incr, which is the with of our
       approx. rectangle. */
    sum = sum + (eval_product(td, x) * bridge_incr);
    
    /* Increment the value of x for the next time through the loop. */
    x = x + bridge_incr;
  }


  /* Now, we see how many cycles we completed in this first bridge_val
     area.  If that number is less than num_cycles, we complete the
     num_cycles, with num_samps_per_cycle in each. */
  x = x - bridge_incr + incr;
  while (x <= max_val){

    /* The area of our approx. rectangle is the approximate height of
       the curve at x times the incr, which is the with of our
       approx. rectangle. */
    sum = sum + (eval_product(td, x) * incr);
    
    /* Increment the value of x for the next time through the loop. */
    x = x + incr;
  }


  /* Constant part (part in front of the integral) is 2*pi*sqrt(2*pi). */
  const_part = 2.0 * M_PI * sqrt(2 * M_PI);

  return(const_part * sum);
}



double eval_product(double td, double x){

  double func_to_transf, eval_bessel;
  

  /* Compute the function we're trying to take the Hankel
     transform of. */
  func_to_transf = pow( 1.0 + gsl_pow_2(2.0 * M_PI * x), -0.75 );
  
  /* Evaluate the Bessel function part, which is where the distance
     td comes in. */
  eval_bessel = gsl_sf_bessel_J0(2.0 * M_PI * x * td );
  
  
  /* Return the product */
  return(x * func_to_transf * eval_bessel);
}
