## ####################################################
## This code requires the R packages:
##   geoR: to draw scatterplots
##   fields: to obtain color scheme (from tim.colors())
## ####################################################


num.time.steps <- 4


## ###########################################
## READ IN THE KERNEL TYPES THAT WE EVENTUALLY MIGHT USE.

source("../../../simul_series/kernel_func.r")

## We use an isotropic version of the GAUSSIAN with an eff.range of 700.
eff.range <- 700
eff.radius <- find.radius.given.eff.range(eff.range, kernel.type="X")
chosen.kernel.df <- data.frame(kernel.type="X", a=eff.radius, b=eff.radius, theta=0.0)
num.kernels <- 1
## ###########################################


## ###########################################
## READ IN OBSERVATION POINTS AND GRID POINTS, DETERMINE BOUNDARIES.

## Read in observations.
obs.pts <- read.csv("../../region_setup/obs_locations.csv")
num.obs <- nrow(obs.pts)

## Read in grid points.
grid.pts <- read.csv("../../region_setup/grid_locations.csv")
## Begin temp stuff.
## cut.uniq <- unique(grid.pts[,1])[seq(1, length(unique(grid.pts[,1])), by=4)]
## grid.pts <- as.data.frame(expand.grid(cut.uniq, cut.uniq))
## names(grid.pts) <- c("x", "y")
## End temp stuff.
num.grid.pts <- nrow(grid.pts)

## Save unique list of delineation points in both directions for grid.
grid.uniq.x <- unique(grid.pts[,1])
grid.uniq.y <- unique(grid.pts[,2])

## Get grid spacing from prediction locatiosn (assume they're all regular).
grid.space <- sort(grid.uniq.x)[2] - sort(grid.uniq.x)[1]

## We know that the range in x and y directions is the range of the
## grid points, with a buffer of half the grid spacing on each side.
x.rng <- c( min(grid.pts[,1])-(0.5*grid.space), max(grid.pts[,1])+(0.5*grid.space) )
y.rng <- c( min(grid.pts[,2])-(0.5*grid.space), max(grid.pts[,2])+(0.5*grid.space) )
## ###########################################


## ###########################################
## READ IN THETAS AND THETA LATTICE.

theta.lattice <- matrix(scan("../theta_lattice.dat"), ncol=2, byrow=T)
theta <- matrix(scan("../theta.dat"), ncol=num.time.steps, byrow=T)
## ###########################################


## #######################################
## FORM K MATRIX FOR COMBINED GRID AND OBS POINTS.

## Make a matrix of combined observation and grid points.
combined.pts <- as.matrix(rbind(obs.pts, grid.pts))

## For each point i at which the process will be simulated, center the
## kernel at point i, and evaluate its height at each of the white
## noise locations.  Put this information in row i of the K matrix.
## a is length of major axis (assumed to be the horizontal)
## b is length of minor axis (assumed to be the vertical)
## Form a weights matrix for each of the kernels.  This matrix is
## weighted to try to correct for the variance.
K.reweight <- matrix(NA, nrow=c(nrow(combined.pts), ncol=nrow(theta.lattice)))

## For each point j at which the process will be simulated, center
## the kernel at point j, and evaluate its corrected height at each
## of the white noise locations. 
K <- matrix(0.0, nrow=nrow(combined.pts), ncol=nrow(theta.lattice))
{
  if (chosen.kernel.df$kernel.type!="X"){
    for (j in 1:nrow(combined.pts)){
      td <- find.standardized.distance(x=theta.lattice[,1], y=theta.lattice[,2], ctr.pt=combined.pts[j,], chosen.kernel.df$a, chosen.kernel.df$b, chosen.kernel.df$theta)
      K[j,] <- find.kernel.height.given.radius.and.transformed.distance(td=td, r=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type)
    }
  }
  
  else{
    ## Find all transformed distances first.
    td <- matrix(NA, nrow=nrow(combined.pts), ncol=nrow(theta.lattice))
    for (j in 1:nrow(combined.pts))
      td[j,] <- find.standardized.distance(x=theta.lattice[,1], y=theta.lattice[,2], ctr.pt=combined.pts[j,], chosen.kernel.df$a, chosen.kernel.df$b, chosen.kernel.df$theta)

    ## Then, take the maximium distance and pass it into the function.
    K <- find.kernel.height.given.radius.and.transformed.distance(td=td, r=sqrt(chosen.kernel.df$a*chosen.kernel.df$b), kernel.type=chosen.kernel.df$kernel.type)
    ##K[K<0] <- 0.0
  }
}
##rm(td, x.rng, j)


write(t(K), file="K_mat", ncol=ncol(K))
